@@ -101,6 +101,58 @@ int dumpfile_is_split(void)
 	return KDUMP_SPLIT();
 }
 
+int have_crash_notes(int cpu)
+{
+	ulong crash_notes, notes_ptr;
+	char *buf, *p;
+	Elf64_Nhdr *note = NULL;
+
+	if (!readmem(symbol_value("crash_notes"), KVADDR, &crash_notes,
+		     sizeof(crash_notes), "crash_notes", RETURN_ON_ERROR)) {
+		error(WARNING, "cannot read \"crash_notes\"\n");
+		return FALSE;
+	}
+
+	if ((kt->flags & SMP) && (kt->flags & PER_CPU_OFF))
+		notes_ptr = crash_notes + kt->__per_cpu_offset[cpu];
+	else
+		notes_ptr = crash_notes;
+
+	buf = GETBUF(SIZE(note_buf));
+
+	if (!readmem(notes_ptr, KVADDR, buf,
+		     SIZE(note_buf), "note_buf_t", RETURN_ON_ERROR)) {
+		error(WARNING, "cpu %d: cannot read NT_PRSTATUS note\n", cpu);
+		return FALSE;
+	}
+
+	note = (Elf64_Nhdr *)buf;
+	p = buf + sizeof(Elf64_Nhdr);
+
+	if (note->n_type != NT_PRSTATUS) {
+		error(WARNING, "cpu %d: invalid NT_PRSTATUS note (n_type != NT_PRSTATUS)\n", cpu);
+		return FALSE;
+	}
+
+	if (!STRNEQ(p, "CORE")) {
+		error(WARNING, "cpu %d: invalid NT_PRSTATUS note (name != \"CORE\")\n", cpu);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+int
+diskdump_is_cpu_prstatus_valid(int cpu)
+{
+	static int crash_notes_exists = -1;
+
+	if (crash_notes_exists == -1)
+		crash_notes_exists = kernel_symbol_exists("crash_notes");
+
+	return (!crash_notes_exists || have_crash_notes(cpu));
+}
+
 void
 map_cpus_to_prstatus_kdump_cmprs(void)
 {
@@ -131,7 +183,7 @@ map_cpus_to_prstatus_kdump_cmprs(void)
 	nrcpus = (kt->kernel_NR_CPUS ? kt->kernel_NR_CPUS : NR_CPUS);
 
 	for (i = 0, j = 0; i < nrcpus; i++) {
-		if (in_cpu_map(ONLINE_MAP, i)) {
+		if (in_cpu_map(ONLINE_MAP, i) && machdep->is_cpu_prstatus_valid(i)) {
 			dd->nt_prstatus_percpu[i] = nt_ptr[j++];
 			dd->num_prstatus_notes = 
 				MAX(dd->num_prstatus_notes, i+1);
@@ -515,16 +567,22 @@ arm_kdump_header_adjust(int header_version)
 static int
 read_pd(int fd, off_t offset, page_desc_t *pd)
 {
-	const off_t failed = (off_t)-1;
+	int ret;
 
 	if (FLAT_FORMAT()) {
 		if (!read_flattened_format(fd, offset, pd, sizeof(*pd)))
 			return READ_ERROR;
 	} else {
-		if (lseek(fd, offset, SEEK_SET) == failed)
+		if (offset < 0) {
+			if (CRASHDEBUG(8))
+				fprintf(fp, "read_pd: invalid offset: %lx\n", offset);
 			return SEEK_ERROR;
-		if (read(fd, pd, sizeof(*pd)) != sizeof(*pd))
+		}
+		if ((ret = pread(fd, pd, sizeof(*pd), offset)) != sizeof(*pd)) {
+			if (ret == -1 && CRASHDEBUG(8))
+				fprintf(fp, "read_pd: pread error: %s\n", strerror(errno));
 			return READ_ERROR;
+		}
 	}
 
 	return 0;
@@ -622,6 +680,9 @@ restart:
 	else if (STRNEQ(header->utsname.machine, "aarch64") &&
 	    machine_type_mismatch(file, "ARM64", NULL, 0))
 		goto err;
+	else if (STRNEQ(header->utsname.machine, "riscv64") &&
+	    machine_type_mismatch(file, "RISCV64", NULL, 0))
+		goto err;
 
 	if (header->block_size != block_size) {
 		block_size = header->block_size;
@@ -780,6 +841,8 @@ restart:
 		dd->machine_type = EM_AARCH64;
 	else if (machine_type("SPARC64"))
 		dd->machine_type = EM_SPARCV9;
+	else if (machine_type("RISCV64"))
+		dd->machine_type = EM_RISCV;
 	else {
 		error(INFO, "%s: unsupported machine type: %s\n", 
 			DISKDUMP_VALID() ? "diskdump" : "compressed kdump",
@@ -1022,6 +1085,7 @@ diskdump_init(char *unused, FILE *fptr)
 	if (!DISKDUMP_VALID() && !KDUMP_CMPRS_VALID())
 		return FALSE;
 
+	machdep->is_cpu_prstatus_valid = diskdump_is_cpu_prstatus_valid;
 	dd->ofp = fptr;
 	return TRUE;
 }
@@ -1120,7 +1184,6 @@ cache_page(physaddr_t paddr)
 	off_t seek_offset;
 	page_desc_t pd;
 	const int block_size = dd->block_size;
-	const off_t failed = (off_t)-1;
 	ulong retlen;
 #ifdef ZSTD
 	static ZSTD_DCtx *dctx = NULL;
@@ -1185,10 +1248,18 @@ cache_page(physaddr_t paddr)
 			return PAGE_INCOMPLETE;
 		}
 	} else {
-		if (lseek(dd->dfd, pd.offset, SEEK_SET) == failed)
+		if (pd.offset < 0) {
+			if (CRASHDEBUG(8))
+				fprintf(fp, "read_diskdump/cache_page: invalid offset: %lx\n",
+					pd.offset);
 			return SEEK_ERROR;
-		if (read(dd->dfd, dd->compressed_page, pd.size) != pd.size)
+		}
+		if ((ret = pread(dd->dfd, dd->compressed_page, pd.size, pd.offset)) != pd.size) {
+			if (ret == -1 && CRASHDEBUG(8))
+				fprintf(fp, "read_diskdump/cache_page: pread error: %s\n",
+					strerror(errno));
 			return READ_ERROR;
+		}
 	}
 
 	if (pd.flags & DUMP_DH_COMPRESSED_ZLIB) {
@@ -1526,6 +1597,12 @@ get_diskdump_regs_mips(struct bt_info *bt, ulong *eip, ulong *esp)
 	machdep->get_stack_frame(bt, eip, esp);
 }
 
+static void
+get_diskdump_regs_riscv64(struct bt_info *bt, ulong *eip, ulong *esp)
+{
+	machdep->get_stack_frame(bt, eip, esp);
+}
+
 static void
 get_diskdump_regs_sparc64(struct bt_info *bt, ulong *eip, ulong *esp)
 {
@@ -1605,6 +1682,10 @@ get_diskdump_regs(struct bt_info *bt, ulong *eip, ulong *esp)
 		get_diskdump_regs_sparc64(bt, eip, esp);
 		break;
 
+	case EM_RISCV:
+		get_diskdump_regs_riscv64(bt, eip, esp);
+		break;
+
 	default:
 		error(FATAL, "%s: unsupported machine type: %s\n",
 			DISKDUMP_VALID() ? "diskdump" : "compressed kdump",
@@ -1751,7 +1832,8 @@ dump_note_offsets(FILE *fp)
 			qemu = FALSE;
 			if (machine_type("X86_64") || machine_type("S390X") ||
 			    machine_type("ARM64") || machine_type("PPC64") ||
-			    machine_type("SPARC64") || machine_type("MIPS64")) {
+			    machine_type("SPARC64") || machine_type("MIPS64") ||
+			    machine_type("RISCV64")) {
 				note64 = (void *)dd->notes_buf + tot;
 				len = sizeof(Elf64_Nhdr);
 				if (STRNEQ((char *)note64 + len, "QEMU"))
@@ -2558,7 +2640,8 @@ dump_registers_for_compressed_kdump(void)
 	if (!KDUMP_CMPRS_VALID() || (dd->header->header_version < 4) ||
 	    !(machine_type("X86") || machine_type("X86_64") ||
 	      machine_type("ARM64") || machine_type("PPC64") ||
-	      machine_type("MIPS") || machine_type("MIPS64")))
+	      machine_type("MIPS") || machine_type("MIPS64") ||
+	      machine_type("RISCV64")))
 		error(FATAL, "-r option not supported for this dumpfile\n");
 
 	if (machine_type("ARM64") && (kt->cpus != dd->num_prstatus_notes))
@@ -2672,15 +2755,36 @@ diskdump_device_dump_info(FILE *ofp)
 	}
 }
 
+static ulong ZRAM_FLAG_SHIFT;
+static ulong ZRAM_FLAG_SAME_BIT;
+
 static void
 zram_init(void)
 {
+	long zram_flag_shift;
+
 	MEMBER_OFFSET_INIT(zram_mempoll, "zram", "mem_pool");
 	MEMBER_OFFSET_INIT(zram_compressor, "zram", "compressor");
 	MEMBER_OFFSET_INIT(zram_table_flag, "zram_table_entry", "flags");
 	if (INVALID_MEMBER(zram_table_flag))
 		MEMBER_OFFSET_INIT(zram_table_flag, "zram_table_entry", "value");
 	STRUCT_SIZE_INIT(zram_table_entry, "zram_table_entry");
+	MEMBER_OFFSET_INIT(zspoll_size_class, "zs_pool", "size_class");
+	MEMBER_OFFSET_INIT(size_class_size, "size_class", "size");
+	MEMBER_OFFSET_INIT(zspage_huge, "zspage", "huge");
+
+	if (enumerator_value("ZRAM_LOCK", &zram_flag_shift))
+		;
+	else if (THIS_KERNEL_VERSION >= LINUX(6,1,0))
+		zram_flag_shift = PAGESHIFT() + 1;
+	else
+		zram_flag_shift = 24;
+
+	ZRAM_FLAG_SHIFT = 1 << zram_flag_shift;
+	ZRAM_FLAG_SAME_BIT = 1 << (zram_flag_shift+1);
+
+	if (CRASHDEBUG(1))
+		fprintf(fp, "zram_flag_shift: %ld\n", zram_flag_shift);
 }
 
 static unsigned char *
@@ -2688,9 +2792,11 @@ zram_object_addr(ulong pool, ulong handle, unsigned char *zram_buf)
 {
 	ulong obj, off, class, page, zspage;
 	struct zspage zspage_s;
+	struct zspage_5_17 zspage_5_17_s;
 	physaddr_t paddr;
 	unsigned int obj_idx, class_idx, size;
 	ulong pages[2], sizes[2];
+	ulong zs_magic;
 
 	readmem(handle, KVADDR, &obj, sizeof(void *), "zram entry", FAULT_ON_ERROR);
 	obj >>= OBJ_TAG_BITS;
@@ -2699,11 +2805,20 @@ zram_object_addr(ulong pool, ulong handle, unsigned char *zram_buf)
 
 	readmem(page + OFFSET(page_private), KVADDR, &zspage,
 			sizeof(void *), "page_private", FAULT_ON_ERROR);
-	readmem(zspage, KVADDR, &zspage_s, sizeof(struct zspage), "zspage", FAULT_ON_ERROR);
 
-	class_idx = zspage_s.class;
-	if (zspage_s.magic != ZSPAGE_MAGIC)
-		error(FATAL, "zspage magic incorrect: %x\n", zspage_s.magic);
+	if (VALID_MEMBER(zspage_huge)) {
+		readmem(zspage, KVADDR, &zspage_5_17_s,
+			sizeof(struct zspage_5_17), "zspage_5_17", FAULT_ON_ERROR);
+		class_idx = zspage_5_17_s.class;
+		zs_magic = zspage_5_17_s.magic;
+	} else {
+		readmem(zspage, KVADDR, &zspage_s, sizeof(struct zspage), "zspage", FAULT_ON_ERROR);
+		class_idx = zspage_s.class;
+		zs_magic = zspage_s.magic;
+	}
+
+	if (zs_magic != ZSPAGE_MAGIC)
+		error(FATAL, "zspage magic incorrect: %x\n", zs_magic);
 
 	class = pool + OFFSET(zspoll_size_class);
 	class += (class_idx * sizeof(void *));
@@ -2721,8 +2836,13 @@ zram_object_addr(ulong pool, ulong handle, unsigned char *zram_buf)
 	}
 
 	pages[0] = page;
-	readmem(page + OFFSET(page_freelist), KVADDR, &pages[1],
+	if (VALID_MEMBER(page_freelist))
+		readmem(page + OFFSET(page_freelist), KVADDR, &pages[1],
 			sizeof(void *), "page_freelist", FAULT_ON_ERROR);
+	else
+		readmem(page + OFFSET(page_index), KVADDR, &pages[1],
+			sizeof(void *), "page_index", FAULT_ON_ERROR);
+
 	sizes[0] = PAGESIZE() - off;
 	sizes[1] = size - sizes[0];
 	if (!is_page_ptr(pages[0], &paddr)) {
@@ -2739,9 +2859,13 @@ zram_object_addr(ulong pool, ulong handle, unsigned char *zram_buf)
 	readmem(paddr, PHYSADDR, zram_buf + sizes[0], sizes[1], "zram buffer[1]", FAULT_ON_ERROR);
 
 out:
-	readmem(page, KVADDR, &obj, sizeof(void *), "page flags", FAULT_ON_ERROR);
-	if (!(obj & (1<<10))) { //PG_OwnerPriv1 flag
-		return (zram_buf + ZS_HANDLE_SIZE);
+	if (VALID_MEMBER(zspage_huge)) {
+		if (!zspage_5_17_s.huge)
+			return (zram_buf + ZS_HANDLE_SIZE);
+	} else {
+		readmem(page, KVADDR, &obj, sizeof(void *), "page flags", FAULT_ON_ERROR);
+		if (!(obj & (1<<10))) // PG_OwnerPriv1 flag
+			return (zram_buf + ZS_HANDLE_SIZE);
 	}
 
 	return zram_buf;
@@ -2920,7 +3044,13 @@ try_zram_decompress(ulonglong pte_val, unsigned char *buf, ulong len, ulonglong
 	readmem(zram_table_entry + OFFSET(zram_table_flag), KVADDR, &flags,
 		sizeof(void *), "zram_table_flag", FAULT_ON_ERROR);
 	if (!entry || (flags & ZRAM_FLAG_SAME_BIT)) {
-		memset(buf, entry, len);
+		int count;
+		ulong *same_buf = (ulong *)GETBUF(PAGESIZE());
+		for (count = 0; count < PAGESIZE() / sizeof(ulong); count++) {
+			same_buf[count] = entry;
+		}
+		memcpy(buf, same_buf + off, len);
+		FREEBUF(same_buf);
 		goto out;
 	}
 	size = flags & (ZRAM_FLAG_SHIFT -1);
