@@ -17,18 +17,21 @@
 
 #include "defs.h"
 
+#ifndef GDB_10_2
 static void exit_after_gdb_info(void);
+#endif
 static int is_restricted_command(char *, ulong);
 static void strip_redirection(char *);
 int get_frame_offset(ulong);
 
 int *gdb_output_format;
 unsigned int *gdb_print_max;
-int *gdb_prettyprint_structs;
-int *gdb_prettyprint_arrays;
-int *gdb_repeat_count_threshold;
-int *gdb_stop_print_at_null;
+unsigned char *gdb_prettyprint_structs;
+unsigned char *gdb_prettyprint_arrays;
+unsigned int *gdb_repeat_count_threshold;
+unsigned char *gdb_stop_print_at_null;
 unsigned int *gdb_output_radix;
+static void gdb_error_debug(void);
 
 static ulong gdb_user_print_option_address(char *);
 
@@ -68,10 +71,12 @@ gdb_main_loop(int argc, char **argv)
 	}
 
         optind = 0;
+#ifndef GDB_10_2
 #if defined(GDB_5_3) || defined(GDB_6_0) || defined(GDB_6_1)
         command_loop_hook = main_loop;
 #else
 	deprecated_command_loop_hook = main_loop;
+#endif
 #endif
         gdb_main_entry(argc, argv);
 }
@@ -117,22 +122,26 @@ void
 display_gdb_banner(void)
 {
 	optind = 0;
+#ifndef GDB_10_2
 #if defined(GDB_5_3) || defined(GDB_6_0) || defined(GDB_6_1)
         command_loop_hook = exit_after_gdb_info;
 #else
         deprecated_command_loop_hook = exit_after_gdb_info;
+#endif
 #endif
 	args[0] = "gdb";
 	args[1] = "-version";
 	gdb_main_entry(2, args);
 }
 
+#ifndef GDB_10_2
 static void
 exit_after_gdb_info(void)
 {
         fprintf(fp, "\n");
         clean_exit(0);
 }
+#endif
 
 /* 
  *  Stash a copy of the gdb version locally.  This can be called before
@@ -186,13 +195,13 @@ gdb_session_init(void)
 		gdb_user_print_option_address("output_format");
 	gdb_print_max = (unsigned int *)
 		gdb_user_print_option_address("print_max");
-	gdb_prettyprint_structs = (int *)
+	gdb_prettyprint_structs = (unsigned char *)
 		gdb_user_print_option_address("prettyprint_structs");
-	gdb_prettyprint_arrays = (int *)
+	gdb_prettyprint_arrays = (unsigned char *)
 		gdb_user_print_option_address("prettyprint_arrays");
-	gdb_repeat_count_threshold = (int *)
+	gdb_repeat_count_threshold = (unsigned int *)
 		gdb_user_print_option_address("repeat_count_threshold");
-	gdb_stop_print_at_null = (int *)
+	gdb_stop_print_at_null = (unsigned char *)
 		gdb_user_print_option_address("stop_print_at_null");
 	gdb_output_radix = (unsigned int *)
 		gdb_user_print_option_address("output_radix");
@@ -291,6 +300,19 @@ retry:
 	sprintf(req->buf, "set width 0");
 	gdb_interface(req);
 
+#ifdef GDB_10_2
+	req->command = GNU_PASS_THROUGH;
+	req->name = NULL, req->flags = 0;
+	sprintf(req->buf, "set max-value-size unlimited");
+	gdb_interface(req);
+
+	req->command = GNU_PASS_THROUGH;
+	req->name = NULL, req->flags = 0;
+	sprintf(req->buf, "set max-completions unlimited");
+	gdb_interface(req);
+#endif
+
+#if 0
        /*
         *  Patch gdb's symbol values with the correct values from either
         *  the System.map or non-debug vmlinux, whichever is in effect.
@@ -303,6 +325,9 @@ retry:
         	if (req->flags & GNU_COMMAND_FAILED)
 			error(FATAL, "patching of gdb symbol values failed\n");
 	} else if (!(pc->flags & SILENT))
+#else
+        if (!(pc->flags & SILENT))
+#endif
 		fprintf(fp, "\n");
 
 
@@ -364,19 +389,6 @@ gdb_interface(struct gnu_request *req)
 	pc->cur_req = req;
 	pc->cur_gdb_cmd = req->command;
 
-	if (req->flags & GNU_RETURN_ON_ERROR) {
-		error_hook = gdb_error_hook;
-        	if (setjmp(pc->gdb_interface_env)) {
-			pc->last_gdb_cmd = pc->cur_gdb_cmd;
-			pc->cur_gdb_cmd = 0;
-			pc->cur_req = NULL;
-			req->flags |= GNU_COMMAND_FAILED;
-			pc->flags &= ~IN_GDB;
-			return;
-		}
-	} else
-		error_hook = NULL;
-
 	if (CRASHDEBUG(2))
 		dump_gnu_request(req, IN_GDB);
 
@@ -400,10 +412,12 @@ gdb_interface(struct gnu_request *req)
 	SIGACTION(SIGINT, restart, &pc->sigaction, NULL);
 	SIGACTION(SIGSEGV, SIG_DFL, &pc->sigaction, NULL);
 
+        if (req->flags & GNU_COMMAND_FAILED)
+                gdb_error_debug();
+
 	if (CRASHDEBUG(2))
 		dump_gnu_request(req, !IN_GDB);
 
-	error_hook = NULL;
         pc->last_gdb_cmd = pc->cur_gdb_cmd;
         pc->cur_gdb_cmd = 0;
 	pc->cur_req = NULL;
@@ -627,8 +641,6 @@ restore_gdb_sanity(void)
         *gdb_prettyprint_structs = 1;   /* these may piss somebody off... */
 	*gdb_repeat_count_threshold = 0x7fffffff;
 
-	error_hook = NULL;
-
 	if (st->flags & ADD_SYMBOL_FILE) {
 		error(INFO, 
 		    "%s\n     gdb add-symbol-file command failed\n", 
@@ -698,11 +710,10 @@ static char *prohibited_list[] = {
 	"run", "r", "break", "b", "tbreak", "hbreak", "thbreak", "rbreak",
 	"watch", "rwatch", "awatch", "attach", "continue", "c", "fg", "detach", 
 	"finish", "handle", "interrupt", "jump", "kill", "next", "nexti", 
-	"signal", "step", "s", "stepi", "target", "thread", "until", "delete", 
-	"clear", "disable", "enable", "condition", "ignore", "frame", 
-	"select-frame", "f", "up", "down", "catch", "tcatch", "return",
-	"file", "exec-file", "core-file", "symbol-file", "load", "si", "ni", 
-	"shell", 
+	"signal", "step", "s", "stepi", "target", "until", "delete", 
+	"clear", "disable", "enable", "condition", "ignore", "frame", "catch",
+	"tcatch", "return", "file", "exec-file", "core-file", "symbol-file",
+	"load", "si", "ni", "shell", "sy",
 	NULL  /* must be last */
 };
 
@@ -742,13 +753,6 @@ is_restricted_command(char *cmd, ulong flags)
 		}
 	}
 
-	if (kt->relocate && 
-	    STRNEQ("disassemble", cmd) && STRNEQ(cmd, "disas"))
-               	error(FATAL, 
-		    "the gdb \"disassemble\" command is prohibited because the kernel text\n"
-		    "%swas relocated%s; use the crash \"dis\" command instead.\n",
-			space(strlen(pc->curcmd)+2), kt->flags2 & KASLR ? " by KASLR" : "");
-	
 	return FALSE;
 }
 
@@ -824,7 +828,6 @@ int
 gdb_readmem_callback(ulong addr, void *buf, int len, int write)
 { 
 	char locbuf[SIZEOF_32BIT], *p1;
-	uint32_t *p2;
 	int memtype;
 	ulong readflags;
 
@@ -881,19 +884,12 @@ gdb_readmem_callback(ulong addr, void *buf, int len, int write)
 		}
 
 		p1 = (char *)buf;
-		if ((memtype == KVADDR) && 
-		    text_value_cache_byte(addr, (unsigned char *)p1)) 
-			return TRUE;
 
 		if (!readmem(addr, memtype, locbuf, SIZEOF_32BIT,
 		    "gdb_readmem_callback", readflags)) 
 			return FALSE;
 
 		*p1 = locbuf[0];
-		if (memtype == KVADDR) {
-			p2 = (uint32_t *)locbuf;
-			text_value_cache(addr, *p2, 0);
-		}
 		return TRUE;
 
 	case SIZEOF_32BIT:
@@ -903,16 +899,10 @@ gdb_readmem_callback(ulong addr, void *buf, int len, int write)
 				return TRUE;
 		}
 
-		if ((memtype == KVADDR) && text_value_cache(addr, 0, buf)) 
-			return TRUE;
-
 		if (!readmem(addr, memtype, buf, SIZEOF_32BIT, 
 		    "gdb_readmem callback", readflags))
 			return FALSE;
 
-		if (memtype == KVADDR)
-			text_value_cache(addr, 
-				(uint32_t)*((uint32_t *)buf), NULL);
 		return TRUE;
 	}
 
@@ -945,11 +935,23 @@ gdb_print_callback(ulong addr)
 		return IS_KVADDR(addr);
 }
 
+char *
+gdb_lookup_module_symbol(ulong addr, ulong *offset)
+{
+	struct syment *sp;
+
+	if ((sp = value_search_module(addr, offset))) {
+		return sp->name;
+	} else {
+		return NULL;
+	}
+}
+
 /*
  *  Used by gdb_interface() to catch gdb-related errors, if desired.
  */
-void
-gdb_error_hook(void)
+static void
+gdb_error_debug(void)
 {
 	char buf1[BUFSIZE];
 	char buf2[BUFSIZE];
@@ -969,13 +971,6 @@ gdb_error_hook(void)
 			gdb_command_string(pc->cur_gdb_cmd, buf1, TRUE), buf2);
 	}
 
-#ifdef GDB_7_6
-	do_cleanups(all_cleanups()); 
-#else
-	do_cleanups(NULL); 
-#endif
-
-	longjmp(pc->gdb_interface_env, 1);
 }
 
 
@@ -1012,28 +1007,30 @@ gdb_set_crash_scope(ulong vaddr, char *arg)
 	char name[BUFSIZE];
 	struct load_module *lm;
 
-	if (!is_kernel_text(vaddr)) {
-		error(INFO, "invalid text address: %s\n", arg);
-		return FALSE;
-	}
+	if (vaddr) {
+		if (!is_kernel_text(vaddr)) {
+			error(INFO, "invalid text address: %s\n", arg);
+			return FALSE;
+		}
 
-	if (module_symbol(vaddr, NULL, &lm, name, 0)) {
-		if (!(lm->mod_flags & MOD_LOAD_SYMS)) {
-			error(INFO, "attempting to find/load \"%s\" module debuginfo\n", 
-				lm->mod_name);
-			if (!load_module_symbols_helper(lm->mod_name)) {
-				error(INFO, "cannot find/load \"%s\" module debuginfo\n", 
+		if (module_symbol(vaddr, NULL, &lm, name, 0)) {
+			if (!(lm->mod_flags & MOD_LOAD_SYMS)) {
+				error(INFO, "attempting to find/load \"%s\" module debuginfo\n",
 					lm->mod_name);
-				return FALSE;
+				if (!load_module_symbols_helper(lm->mod_name)) {
+					error(INFO, "cannot find/load \"%s\" module debuginfo\n",
+						lm->mod_name);
+					return FALSE;
+				}
 			}
 		}
-	} else if (kt->flags2 & KASLR)
-		vaddr -= (kt->relocate * -1);
+	}
 
 	req->command = GNU_SET_CRASH_BLOCK;
 	req->addr = vaddr;
 	req->flags = 0;
 	req->addr2 = 0;
+	req->fp = pc->nullfp;
 	gdb_command_funnel(req);    
 
 	if (CRASHDEBUG(1))
@@ -1063,4 +1060,37 @@ get_frame_offset(ulong pc)
 }
 #endif /* !ALPHA */ 
 
+unsigned long crash_get_kaslr_offset(void);
+unsigned long crash_get_kaslr_offset(void)
+{
+        return kt->relocate * -1;
+}
+
+/* Callbacks for crash_target */
+int crash_get_nr_cpus(void);
+int crash_get_cpu_reg (int cpu, int regno, const char *regname,
+                       int regsize, void *val);
+
+int crash_get_nr_cpus(void)
+{
+        if (SADUMP_DUMPFILE())
+                return sadump_get_nr_cpus();
+        else if (DISKDUMP_DUMPFILE())
+                return diskdump_get_nr_cpus();
+        else if (KDUMP_DUMPFILE())
+                return kdump_get_nr_cpus();
+        else if (VMSS_DUMPFILE())
+                return vmware_vmss_get_nr_cpus();
+
+        /* Just CPU #0 */
+        return 1;
+}
+
+int crash_get_cpu_reg (int cpu, int regno, const char *regname,
+                       int regsize, void *value)
+{
+        if (!machdep->get_cpu_reg)
+                return FALSE;
+        return machdep->get_cpu_reg(cpu, regno, regname, regsize, value);
+}
 
